package com.syc.slm.slmbi.controller;

import com.syc.slm.common.core.util.R;
import com.syc.slm.slmbi.dto.CreateInstrumentPanelGroupDTO;
import com.syc.slm.slmbi.dto.InstrumentPanelGroupDTO;
import com.syc.slm.slmbi.service.InstrumentPanelGroupService;
import com.syc.slm.slmbi.vo.InstrumentPanelGroupVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * slm-bi
 * 仪表板分组
 *
 * @author : lin
 * @date : 2021-03-25 15:07
 **/
@RestController
@RequestMapping("/{appId}/groups")
@Api(value = "仪表板分组", tags = "仪表板分组")
public class InstrumentPanelGroupController extends BaseRestController{

    @Autowired
    private InstrumentPanelGroupService instrumentPanelGroupService;


    @GetMapping
    @ApiOperation("仪表板列表查询，仪表板分组增删改页面的分组树")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class),
        @ApiImplicitParam(name = "name", value = "分组名称",dataTypeClass = String.class)
    })
    public R<List<InstrumentPanelGroupVo>> selectGroupList(@PathVariable String appId, @RequestParam(required = false,value = "name") String name) {
        return call(()->instrumentPanelGroupService.selectGroupList(appId,name));
    }

    @PostMapping
    @ApiOperation("保存仪表板分组")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<String> saveGroup(@PathVariable String appId, @RequestBody CreateInstrumentPanelGroupDTO group) {
        return call(()->instrumentPanelGroupService.saveGroup(group,appId));
    }

    @PutMapping
    @ApiOperation("更新仪表板分组(一级分组是初始化得，不允许修改，其他分组修改必须传parentId)")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<String> updateGroup(@PathVariable String appId, @RequestBody InstrumentPanelGroupDTO group) {
        return call(()->instrumentPanelGroupService.updateGroup(group,appId));
    }

    @DeleteMapping("/{groupId}")
    @ApiOperation("删除仪表板分组")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<String> delGroup(@PathVariable String appId, @PathVariable String groupId) {
        return call(()->instrumentPanelGroupService.delGroup(groupId,false));
    }

}
