package com.syc.slm.slmbi.controller;

import com.syc.slm.common.core.util.R;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.service.InstrumentPanelConfigService;
import com.syc.slm.slmbi.service.InstrumentPanelTreeService;
import com.syc.slm.slmbi.vo.InstrumentPanelConfigVo;
import com.syc.slm.slmbi.vo.PanelTreeNodeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * @author liangbin
 * @date 2021/3/31
 */
@RestController
@RequestMapping("/{appId}/panel-tree")
@Api(value = "仪表板树接口", tags = "仪表板树接口")
public class InstrumentPanelTreeController extends BaseRestController {

    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private InstrumentPanelConfigService configService;

    @GetMapping
    @ApiOperation("显示已发布仪表板树==》(刚进入页面)")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class)
    public R<List<PanelTreeNodeVo>> loadTrees(HttpServletRequest request,@PathVariable String appId) {
        return call(() -> {
            CurrentUser currentUser = getCurrentUser(request);
            return treeService.loadTrees(appId,currentUser);
        });
    }

    @GetMapping("/search")
    @ApiOperation("显示已发布仪表板树==》(点击查询)")
    @ApiImplicitParams({@ApiImplicitParam(name = "appId",
        value = "租户id",
        required = true,
        dataTypeClass = String.class),
        @ApiImplicitParam(name = "name", value = "分组名称或者仪表板名称", dataTypeClass = String.class)})
    public R<List<PanelTreeNodeVo>> loadTrees(HttpServletRequest request, @PathVariable String appId,
        @RequestParam(value = "name", required = false) String name) {
        return call(() -> {
            CurrentUser currentUser = getCurrentUser(request);
            return treeService.loadTrees(appId, name,  currentUser);
        });
    }

    @GetMapping("/children")
    @ApiOperation("点击节点，显示已发布仪表板树")
    @ApiImplicitParams({@ApiImplicitParam(name = "appId",
        value = "租户id",
        required = true,
        dataTypeClass = String.class),
        @ApiImplicitParam(name = "nodeId", value = "分组id", required = true, dataTypeClass = String.class),
        @ApiImplicitParam(name = "name", value = "分组名称或者仪表板名称", dataTypeClass = String.class)})
    public R<List<PanelTreeNodeVo>> children(HttpServletRequest request,@PathVariable String appId,
        @RequestParam("nodeId") String nodeId,
        @RequestParam(value = "name", required = false) String name) {
        return call(() -> {
            CurrentUser currentUser = getCurrentUser(request);
            return treeService.children(nodeId, name,currentUser);
        });
    }

    @GetMapping("/children-panel")
    @ApiOperation("点击节点，显示已发布仪表板配置")
    @ApiImplicitParams({@ApiImplicitParam(name = "appId",
        value = "租户id",
        required = true,
        dataTypeClass = String.class),
        @ApiImplicitParam(name = "panelId", value = "仪表板id", required = true, dataTypeClass = String.class)})
    public R<InstrumentPanelConfigVo> childrenPanel(@PathVariable String appId, @RequestParam("panelId") String panelId) {
        return call(() -> (configService.getPublishConfigByPanelId(panelId)));
    }


    @GetMapping("/copy")
    @ApiOperation("复制仪表板配置树")
    @ApiImplicitParams({@ApiImplicitParam(name = "appId",
        value = "租户id",
        required = true,
        dataTypeClass = String.class),
        @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class),
        @ApiImplicitParam(name = "name", value = "分组名称或者仪表板名称", dataTypeClass = String.class)})
    public R<List<PanelTreeNodeVo>> copyTreeList(@PathVariable String appId,@RequestParam(value = "name", required = false) String name) {
        return call(() -> (treeService.copyTreeList(appId,name)));
    }

    @GetMapping("/cite")
    @ApiOperation("复用仪表板树")
    @ApiImplicitParams({@ApiImplicitParam(name = "appId",
        value = "租户id",
        required = true,
        dataTypeClass = String.class),
        @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class),
        @ApiImplicitParam(name = "name", value = "分组名称，仪表板名称，组件名称", dataTypeClass = String.class)})
    public R<List<PanelTreeNodeVo>> cite(@PathVariable String appId,@RequestParam(value = "name", required = false) String name) {
        return call(() -> (treeService.citeTreeList(appId,name)));
    }
    @GetMapping("/cite-assembly")
    @ApiOperation("复用仪表板树点击组件获取配置(会返回一个map  优先从key为 wait_published 中匹配，如果找不到再从key 为published 中匹配 )")
    @ApiImplicitParams({@ApiImplicitParam(name = "appId",
        value = "租户id",
        required = true,
        dataTypeClass = String.class),
        @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class),
        @ApiImplicitParam(name = "panelId", value = "仪表板id", required = true, dataTypeClass = String.class)})
    public R<Map<String,String>> assembly(@PathVariable String appId,@RequestParam(value = "panelId", required = false) String panelId) {
        return call(() -> (treeService.assembly(appId,panelId)));
    }
}
