package com.syc.slm.slmbi.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.slmbi.dto.InstrumentPanelGroupDTO;
import com.syc.slm.slmbi.entity.BaseEntity;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.entity.InstrumentPanelGroup;
import com.syc.slm.slmbi.service.InstrumentPanelGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * slm-bi
 * 提供给dc调用
 * @author : lin
 * @date : 2021-05-28 14:22
 **/
@RestController
@RequestMapping("/{appId}/tenant")
@Api(value = "租户", tags = "租户")
@Slf4j
public class TenantController extends BaseRestController{

    @Autowired
    private InstrumentPanelGroupService groupService;
    @PostMapping
    @ApiOperation("新增租户初始化分组")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class)
    public R<String> initGroup(@PathVariable String appId) {
        return call(() -> {
            log.info("盛云平台创建租户初始化分组");
            log.info("appId:"+appId);
            log.info("盛云平台创建租户初始化分组");
            InstrumentPanelGroupDTO groupDTO = new InstrumentPanelGroupDTO();
            groupDTO.setParentId(null);
            groupDTO.setName("全部分组");
           return  groupService.saveGroup(groupDTO,appId);
        });
    }

    @DeleteMapping
    @ApiOperation("删除租户初始化分组数据")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<String> delGroup(HttpServletRequest request, @PathVariable String appId) {
        return call(() -> {
            CurrentUser currentUser = getCurrentUser(request);
            LambdaQueryWrapper<InstrumentPanelGroup> where = new LambdaQueryWrapper<>();
            where.eq(BaseEntity::getAppId, currentUser.getAppId());
            where.eq(InstrumentPanelGroup::getName, "全部分组");
            where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
            InstrumentPanelGroup one = groupService.getOne(where);
            if(ObjectUtils.isNotEmpty(one)) {
                return groupService.delGroup(one.getId(), true);
            }else{
                return null;
            }
        });
    }
}
