package com.syc.slm.slmbi.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.syc.slm.slmbi.dto.InstrumentPanelAssemblyDTO;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.entity.InstrumentPanelTree;
import com.syc.slm.slmbi.vo.PanelTreeNodeVo;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 仪表板树聚合表
 *
 * @author lin
 * @date 2021-04-01 10:52:44
 */
public interface InstrumentPanelTreeService extends IService<InstrumentPanelTree> {

    void batchUpdateTreeByIds(Set<String> treeIds, Integer recordStatus);

    /**
     * 初始化仪表板树
     *
     * @param nodeId
     * @param nodeType
     * @param bizId
     * @param bizName
     */
    void initTree(String nodeId, String nodeType, String bizId, String bizName);

    /**
     * 根据仪表板id删除仪表板树
     *
     * @param panelId
     */
    void delTreeByPanelId(String panelId);

    /**
     * 根据仪表板详情保存仪表板组件树
     *
     * @param tree
     * @param assembly
     * @param panelId
     */
    void savePanelAssemblyTree(InstrumentPanelTree tree, List<InstrumentPanelAssemblyDTO> assembly, String panelId);

    /**
     * 根据fullId 删除仪表板树
     * @param fullId
     */
    void delTreeByPanelFullId(String fullId);

    /**
     * 更新仪表板树
     * @param panelId
     * @param panelName
     * @param groupId
     */
    void updatePanelTreeFullName(String panelId,String panelName,String groupId) ;

    /**
     * 加载树
     * @param appId
     * @param name
     * @return
     */
    List<PanelTreeNodeVo> loadTrees(String appId, String name,CurrentUser user);

    /**
     * 加载树
     * @param appId
     * @return
     */
    List<PanelTreeNodeVo> loadTrees(String appId,CurrentUser user);

    /**
     * 加载树节点下一节点
     * @param groupId
     * @param name
     * @return
     */
    List<PanelTreeNodeVo> children(String groupId, String name,CurrentUser user);

    /**
     * 根据仪表板分组id更新 fullId fullName
     * @param groupId
     * @param panelId
     * @param panelName
     */
    void updateFullNameByGroupId(String groupId, String panelId, String panelName);

    /**
     * 复制仪表板配置显示的仪表板树
     * @param appId
     * @param name
     * @return
     */
    List<PanelTreeNodeVo> copyTreeList(String appId,String name);

    /**
     * 复用仪表板树
     * @param appId
     * @param name
     * @return
     */
    List<PanelTreeNodeVo>  citeTreeList(String appId, String name);

    /**
     * 点击仪表板组件获取对应配置
     * @param appId
     * @param panelId
     * @return
     */
    Map<String, String> assembly(String appId, String panelId);
}
