package com.syc.slm.slmbi.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.syc.slm.slmbi.dto.*;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.vo.VariableDeviceVo;
import com.syc.slm.slmbi.vo.VariableSourceVo;
import com.syc.slm.slmbi.vo.VariableVo;

import java.util.List;
import java.util.Map;

/**
 * slm-bi
 * 变量
 *
 * @author : lin
 * @date : 2021-03-25 16:17
 **/
public interface VariableService {
    /**
     * 获取变量列表
     * @param variableDTO
     * @return
     */
    IPage<VariableVo> getVariableList(VariableDTO variableDTO,String appId,String token);

    /**
     * 获取变量来源
     * @return
     */
    List<VariableSourceVo> getVariableSourceList();

    /**
     * 获取变量所属设备
     * @param token
     * @param appId
     * @param queryPageDTO
     * @return
     */
    IPage<VariableDeviceVo> getVariableDeviceList(String token,String appId , QueryPageDTO queryPageDTO);

    /**
     * 获取实时数据
     * @param variableDTO
     * @param user
     * @return
     */
    Map<String,List<Object>> listVariableAllDataListByTimeInterval(VariableAllDataDTO variableDTO,CurrentUser user);

    /**
     * 获取历史数据
     * @param variableDTO
     * @param currentUser
     * @return
     */
    Map<String,Object> listVariableHistoryDataListByCode(VariableHistoryDataDTO variableDTO,CurrentUser currentUser);

    /**
     * 下发
     * @param sendDTO
     * @param currentUser
     */
    void send(VariableSendDTO sendDTO, CurrentUser currentUser);

    /**
     * 状态点实时数据
     * @param deviceId
     * @param variableCode
     * @return
     */
    Map<String,Object> status(String deviceId,String variableId,String variableCode,CurrentUser user);

    /**
     * 状态点历史数据
     * @param deviceId
     * @param variableCode
     * @param startTime
     * @param endTime
     * @param currentUser
     * @return
     */
    Map<String,Object> historyStatus( String deviceId, String variableCode, String startTime, String endTime, CurrentUser currentUser);

    /**
     * 获取历史双曲线
     * @param variableDTO
     * @param currentUser
     * @return
     */
     // Map<String,List<Map<String,Object>>> getVariableHistoryCurve(VariableHistoryCurveDataDTO variableDTO, CurrentUser currentUser,String uri);

    /**
     * 查询实时数据
     * @param variableDTO 请求参数
     * @param user 当前用户
     * @return 返回值
     */
    Map<String,List<Object>> listRealTimeVariableData( RealTimeVariableDataDTO variableDTO,CurrentUser user);

    /**
     * 批量查询实时状态数据
     * @param statusRealTimeDataList 请求条件
     * @param user 当前用户
     * @return key为设备ID,value中key为变量名称，vlaue是数据
     */
    Map<String,Map<String,Object>> listVariableStatusRealTimeData( List<VariableStatusRealTimeDataDTO> statusRealTimeDataList,CurrentUser user);

}
