package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelConfigMapper;
import com.syc.slm.slmbi.dto.InstrumentPanelConfigDTO;
import com.syc.slm.slmbi.dto.InstrumentPanelPermissionDTO;
import com.syc.slm.slmbi.entity.*;
import com.syc.slm.slmbi.enums.PublishedStatus;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.*;
import com.syc.slm.slmbi.vo.CustomPermissionVo;
import com.syc.slm.slmbi.vo.InstrumentPanelConfigVo;
import com.syc.slm.slmbi.vo.InstrumentPanelVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * slm-bi
 * 仪表板数据以及样式配置
 *
 * @author : lin
 * @date : 2021-03-25 19:11
 **/
@Slf4j
@Service
public class InstrumentPanelConfigServiceImpl extends ServiceImpl<InstrumentPanelConfigMapper, InstrumentPanelConfig>
    implements InstrumentPanelConfigService {

    @Autowired
    private InstrumentPanelPermissionService permissionService;

    @Autowired
    private InstrumentPanelService panelService;

    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private InstrumentPanelDraftService draftService;

    @Autowired
    private InstrumentPanelGroupService panelGroupService;

    @Override
    @Transactional(readOnly = true)
    public InstrumentPanelConfigVo getPublishConfigByPanelId(@NotNull String instrumentPanelId) {
        return getInstrumentPanelConfigVo(instrumentPanelId, PublishedStatus.PUBLISHED);
    }

    @Override
    @Transactional(readOnly = true)
    public InstrumentPanelConfigVo getConfigByPanelId(String instrumentPanelId) {
        LambdaQueryWrapper<InstrumentPanelConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelConfig::getPanelId, instrumentPanelId);
        where.eq(InstrumentPanelConfig::getPublished, PublishedStatus.WAIT_PUBLISHED.value);
        InstrumentPanelConfig config = baseMapper.selectOne(where);
        if(ObjectUtils.isEmpty(config)){
            where = new LambdaQueryWrapper<>();
            where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
            where.eq(InstrumentPanelConfig::getPanelId, instrumentPanelId);
            where.eq(InstrumentPanelConfig::getPublished, PublishedStatus.PUBLISHED.value);
            config = baseMapper.selectOne(where);
        }
        InstrumentPanelConfigVo vo=new InstrumentPanelConfigVo();
        BeanUtils.copyProperties(config, vo);
        if(StringUtils.isNotEmpty(vo.getConfigDetails())) {
            if (ObjectUtils.isNotEmpty(vo)) {
                try {
                    vo.setConfigDetails(URLDecoder.decode(vo.getConfigDetails(), StandardCharsets.UTF_8.name()));
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new SysException("仪表板配置详情解密异常");
                }
            }
        }
        List<CustomPermissionVo> permission = permissionService.getPublishPermissionByPanelId(
            instrumentPanelId,
            PublishedStatus.WAIT_PUBLISHED.value);
        if(ObjectUtils.isEmpty(permission)){
            permission= permissionService.getPublishPermissionByPanelId(
                instrumentPanelId,
                PublishedStatus.PUBLISHED.value);
        }
        vo.setPermissions(permission);
        InstrumentPanel panel = panelService.getById(instrumentPanelId);
        InstrumentPanelVo panelVo = new InstrumentPanelVo();
        BeanUtils.copyProperties(panel, panelVo);
        vo.setPanelId(instrumentPanelId);
        panelVo.setGroupName(panelGroupService.getById(panel.getGroupId()).getName());
        vo.setPanel(panelVo);
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String publish(String appId, InstrumentPanelConfigDTO config) {
        InstrumentPanelPermissionDTO permission = config.getPermission();
        checkAll(appId, config, permission);
        InstrumentPanel panel = panelService.getById(config.getPanelId());
        if (panel == null){
            throw new SysException("仪表盘不存在，请重新选择");
        }
        fillPanelProp(permission, panel, PublishedStatus.PUBLISHED.value);
        log.info("删除仪表板配置详情");
        delConfig(config.getPanelId(), null);
        log.info("保存配置详情");
        savePanelConfig(panel.getId(), config.getConfigDetails(), PublishedStatus.PUBLISHED.value);

        log.info("删除仪表板权限信息");
        permissionService.delPermissionByPanelId(config.getPanelId(), null);
        log.info("保存权限信息");
        permissionService.savePanelPermission(permission.getCustomPermissions(),panel.getId(), permission.getAccessType(),PublishedStatus.PUBLISHED.value);

        log.info("更新仪表板树");
        updateTree(config, panel);

        InstrumentPanelDraft draft = draftService.getByPanelId(panel.getId());
        if (ObjectUtils.isNotEmpty(draft)) {
            String beforeId =draft.getId();
            BeanUtils.copyProperties(panel, draft);
            draft.setId(beforeId);
            draft.setPanelId(panel.getId());
            draft.setAccessPermission(panel.getAccessPermission());
            draftService.updateById(draft);
        } else {
            InstrumentPanelDraft df = new InstrumentPanelDraft();
            BeanUtils.copyProperties(panel, df);
            df.setId(IdWorker.get32UUID());
            df.setPanelId(panel.getId());
            df.setPublished(PublishedStatus.PUBLISHED.value);
            draftService.save(df);
        }

        panelService.updateById(panel);
        return panel.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveConfig(String appId, InstrumentPanelConfigDTO config) {
        InstrumentPanelPermissionDTO permission = config.getPermission();
        checkAll(appId, config, permission);

        InstrumentPanel panel = panelService.getById(config.getPanelId());
        if (panel == null){
            throw new SysException("仪表盘不存在，输入错误");
        }
        fillPanelProp(permission, panel,PublishedStatus.WAIT_PUBLISHED.value);

        //判断是否有发布版本
        InstrumentPanelDraft draft = draftService.getByPanelId(panel.getId());
        if (ObjectUtils.isNotEmpty(draft)) {
            log.info("删除待发布配置详情");
            delConfig(panel.getId(),PublishedStatus.WAIT_PUBLISHED.value);
            log.info("删除仪表板权限信息");
            permissionService.delPermissionByPanelId(config.getPanelId(), PublishedStatus.WAIT_PUBLISHED.value);

            log.info("更新发布版本的groupId");
            draft.setGroupId(permission.getGroupId());
            draftService.updateById(draft);
            panel.setPublished(PublishedStatus.PUBLISHED.value);

        } else {
            //修改
            log.info("删除仪表板配置详情");
            delConfig(config.getPanelId(), null);
            log.info("删除仪表板权限信息");
            permissionService.delPermissionByPanelId(config.getPanelId(), null);


        }
        log.info("保存配置详情");
        savePanelConfig(panel.getId(), config.getConfigDetails(), PublishedStatus.WAIT_PUBLISHED.value);
        log.info("保存权限信息");
        permissionService.savePanelPermission(permission.getCustomPermissions(),panel.getId(),permission.getAccessType(),PublishedStatus.WAIT_PUBLISHED.value);

        log.info("更新仪表板树");
        updateTree(config, panel);
        panelService.updateById(panel);
        return config.getPanelId();
    }

    private void checkAll(String appId, InstrumentPanelConfigDTO config, InstrumentPanelPermissionDTO permission) {
        if (ObjectUtils.isEmpty(permission)) {
            throw new SysException("权限必填");
        }
        if (StringUtils.isEmpty(permission.getPanelName())) {
            throw new SysException("仪表板名字必填");
        }
        if (StringUtils.isEmpty(permission.getGroupId())) {
            throw new SysException("仪表板分组必填");
        }
        log.info("校验仪表板名称是否重复");
        panelService.checkName(appId, config.getPanelId(), permission.getPanelName());
    }

    @Override
    public void delConfig(@NotNull String panelId, String publishedStatus) {
        List<InstrumentPanelConfig> configs = getConfigByPanelIdAndPublishStatus(panelId,publishedStatus);
        if (CollectionUtils.isNotEmpty(configs)) {
            configs.forEach(x -> {
                x.setRecordStatus(SlmConstants.DATA_DELETED);
                baseMapper.updateById(x);
            });
        }
    }

    @Override
    public void savePanelConfig(String panelId, String configDetails, String publishStatus) {
        InstrumentPanelConfig panelConfig = new InstrumentPanelConfig();
        panelConfig.setId(IdWorker.get32UUID());
        panelConfig.setPanelId(panelId);
        panelConfig.setPublished(publishStatus);
        if(StringUtils.isNotEmpty(configDetails)) {
            try {
                panelConfig.setConfigDetails(URLEncoder.encode(configDetails, StandardCharsets.UTF_8.name()));
            } catch (Exception e) {
                e.printStackTrace();
                throw new SysException("仪表板配置详情加密异常");
            }
        }
        baseMapper.insert(panelConfig);
    }

    @Override
    public void publishConfigByPanelIds(List<String> ids) {
        LambdaQueryWrapper<InstrumentPanelConfig> where = new LambdaQueryWrapper<>();
        where.eq(InstrumentPanelConfig::getRecordStatus, SlmConstants.DATA_VALID);
        where.in(InstrumentPanelConfig::getPanelId, ids);
        List<InstrumentPanelConfig> configs = baseMapper.selectList(where);
        if (CollectionUtils.isNotEmpty(configs)) {
            Map<String, List<InstrumentPanelConfig>> configMaps = configs.stream().collect(Collectors.groupingBy(InstrumentPanelConfig::getPanelId));
            if(CollectionUtils.isNotEmpty(configMaps)){
                configMaps.forEach((k,v)->{
                    List<InstrumentPanelConfig> waitPublishConfigs = configs.stream().filter(x -> PublishedStatus.WAIT_PUBLISHED.value.equals(x.getPublished())).collect(Collectors.toList());
                    List<InstrumentPanelConfig> publishConfigs = configs.stream().filter(x -> PublishedStatus.PUBLISHED.value.equals(x.getPublished())).collect(Collectors.toList());
                    if(CollectionUtils.isNotEmpty(waitPublishConfigs)){
                        waitPublishConfigs.forEach(x -> {
                            x.setPublished(PublishedStatus.PUBLISHED.value);
                            baseMapper.updateById(x);
                        });
                        if(CollectionUtils.isNotEmpty(publishConfigs)){
                            publishConfigs.forEach(x -> {
                                x.setRecordStatus(SlmConstants.DATA_DELETED);
                                baseMapper.updateById(x);
                            });
                        }
                    }

                });
            }

        }

    }

    @Override
    @Transactional(readOnly = true)
    public List<InstrumentPanelConfig> getConfigByPanelIdAndPublishStatus(String panelId, String publishStatus) {
        LambdaQueryWrapper<InstrumentPanelConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelConfig::getPanelId, panelId);
        if(publishStatus!=null) {
            where.eq(InstrumentPanelConfig::getPublished, publishStatus);
        }
        return baseMapper.selectList(where);
    }

    @Override
    public void offlineConfig(String panelId, String publishStatus) {
        List<InstrumentPanelConfig> configs = getConfigByPanelIdAndPublishStatus(panelId, publishStatus);

        InstrumentPanelConfig waitConfig = configs.stream()
            .filter(x -> PublishedStatus.WAIT_PUBLISHED.value.equals(x.getPublished()))
            .findFirst()
            .orElse(null);
        if(ObjectUtils.isNotEmpty(waitConfig)){
            InstrumentPanelConfig publishConfig =
                configs.stream().filter(x -> PublishedStatus.PUBLISHED.value.equals(x.getPublished())).findFirst().orElse(null);
            if(ObjectUtils.isNotEmpty(publishConfig)){
                publishConfig.setRecordStatus(SlmConstants.DATA_DELETED);
                baseMapper.updateById(publishConfig);
            }
        }else{
            InstrumentPanelConfig pushConfig = configs.stream()
                .filter(x -> PublishedStatus.PUBLISHED.value.equals(x.getPublished()))
                .findFirst()
                .orElse(null);
            if(ObjectUtils.isNotEmpty(pushConfig)){
                InstrumentPanelConfig config = new InstrumentPanelConfig();
                BeanUtils.copyProperties(pushConfig,config);
                config.setPublished(PublishedStatus.WAIT_PUBLISHED.value);
                config.setId(IdWorker.get32UUID());
                baseMapper.insert(config);
                pushConfig.setRecordStatus(SlmConstants.DATA_DELETED);
                baseMapper.updateById(pushConfig);

            }
        }
    }

    @Override
    @Transactional(readOnly = true)
    public Map<String, String> getAssemblyConfig(@NotNull String panelId) {
        Map<String,String> maps = Maps.newHashMap();
        LambdaQueryWrapper<InstrumentPanelConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelConfig::getPanelId, panelId);
        List<InstrumentPanelConfig> configs = baseMapper.selectList(where);
        String waitPublished =null;
        String published=null;
        if(CollectionUtils.isNotEmpty(configs)){
            InstrumentPanelConfig waitConfig = configs.stream()
                .filter(x -> PublishedStatus.WAIT_PUBLISHED.value.equals(x.getPublished()))
                .findFirst()
                .orElse(null);
            if(ObjectUtils.isNotEmpty(waitConfig)){
                if(StringUtils.isNotEmpty(waitConfig.getConfigDetails())) {
                    try {
                        waitPublished = URLDecoder.decode(waitConfig.getConfigDetails(), StandardCharsets.UTF_8.name());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
            InstrumentPanelConfig publishConfig = configs.stream()
                .filter(x -> PublishedStatus.PUBLISHED.value.equals(x.getPublished()))
                .findFirst()
                .orElse(null);
            if(ObjectUtils.isNotEmpty(publishConfig)){
                if(StringUtils.isNotEmpty(publishConfig.getConfigDetails())) {
                    try {
                        published = URLDecoder.decode(publishConfig.getConfigDetails(), StandardCharsets.UTF_8.name());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
        maps.put(PublishedStatus.WAIT_PUBLISHED.value,waitPublished);
        maps.put(PublishedStatus.PUBLISHED.value,published);
        return maps;
    }

    @Override
    @Transactional
    public Set<String> getConfigByPanelIds(@NotNull Set<String> ids) {
        LambdaQueryWrapper<InstrumentPanelConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.in(InstrumentPanelConfig::getPanelId, ids);
        where.eq(InstrumentPanelConfig::getPublished, PublishedStatus.WAIT_PUBLISHED.value);
        List<InstrumentPanelConfig> configs = baseMapper.selectList(where);
        if(CollectionUtils.isNotEmpty(configs)){
            return configs.stream().map(InstrumentPanelConfig::getPanelId).collect(Collectors.toSet());
        }

        return Sets.newHashSet();
    }

    private InstrumentPanelConfigVo getInstrumentPanelConfigVo(String instrumentPanelId, PublishedStatus published) {

        LambdaQueryWrapper<InstrumentPanelConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelConfig::getPanelId, instrumentPanelId);
        where.eq(InstrumentPanelConfig::getPublished, published.value);
        InstrumentPanelConfig config = baseMapper.selectOne(where);
        InstrumentPanelConfigVo vo=new InstrumentPanelConfigVo();
        if(ObjectUtils.isNotEmpty(config)) {
            BeanUtils.copyProperties(config, vo);
            if (ObjectUtils.isNotEmpty(vo)) {
                try {
                    String configDetails = vo.getConfigDetails();
                    if (StringUtils.isNotBlank(configDetails)){
                        vo.setConfigDetails(URLDecoder.decode(configDetails, StandardCharsets.UTF_8.name()));
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new SysException("仪表板配置详情解密异常");
                }
            }
        }
        vo.setPermissions(permissionService.getPublishPermissionByPanelId(instrumentPanelId, published.value));
        InstrumentPanelVo panelVo = new InstrumentPanelVo();
        if (PublishedStatus.PUBLISHED.value.equals(published.value)) {
            InstrumentPanelDraft panel = draftService.getByPanelId(instrumentPanelId);
            BeanUtils.copyProperties(panel, panelVo);
            panelVo.setGroupName(panelGroupService.getById(panel.getGroupId()).getName());
        } else {
            InstrumentPanel panel = panelService.getById(instrumentPanelId);
            BeanUtils.copyProperties(panel, panelVo);
            vo.setPanelId(instrumentPanelId);
            panelVo.setGroupName(panelGroupService.getById(panel.getGroupId()).getName());
        }
        vo.setPanel(panelVo);
        return vo;
    }

    private void fillPanelProp(InstrumentPanelPermissionDTO permission, InstrumentPanel panel,String publishStatus) {
        panel.setGroupId(permission.getGroupId());
        panel.setName(permission.getPanelName());
        panel.setAccessPermission(permission.getAccessType());
        panel.setDescription(permission.getDescription());
        panel.setPublished(publishStatus);
        panel.setAccessPermission(permission.getAccessType());
    }

    private void updateTree(InstrumentPanelConfigDTO config, InstrumentPanel panel) {
        log.info("更新仪表盘树");
        InstrumentPanelTree tree = treeService.getById(config.getPanelId());
        String beforeFullId = tree.getFullId();
        treeService.updatePanelTreeFullName(panel.getId(), panel.getName(), panel.getGroupId());
        log.info("查找仪表盘配置组件树并删除");
        treeService.delTreeByPanelFullId(beforeFullId);
        log.info("重新构建组件树");
        treeService.savePanelAssemblyTree(tree, config.getAssemblys(), panel.getId());
    }
}
