package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelDraftMapper;
import com.syc.slm.slmbi.entity.BaseEntity;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.entity.InstrumentPanel;
import com.syc.slm.slmbi.entity.InstrumentPanelDraft;
import com.syc.slm.slmbi.service.InstrumentPanelDraftService;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 仪表板草稿
 *
 * @author lin
 * @date 2021-04-01 10:52:44
 */
@Service
public class InstrumentPanelDraftServiceImpl extends ServiceImpl<InstrumentPanelDraftMapper, InstrumentPanelDraft>
    implements InstrumentPanelDraftService {

    @Override
    public void saveDrafts(List<InstrumentPanel> panels) {
        if (CollectionUtils.isNotEmpty(panels)) {
            List<InstrumentPanelDraft> drafts = SlmConstants.gson.fromJson(SlmConstants.gson.toJson(panels), new TypeToken<List<InstrumentPanelDraft>>() {
            }.getType());
            if (CollectionUtils.isNotEmpty(drafts)) {
                List<InstrumentPanelDraft> panelDrafts = getByIds(panels.stream().map(InstrumentPanel::getId).collect(Collectors.toSet()));
                Map<String, String> maps = CollectionUtils.isNotEmpty(panelDrafts) ? panelDrafts.stream().collect(
                    Collectors.toMap(InstrumentPanelDraft::getPanelId, BaseEntity::getId)) : Maps.newHashMap();
                drafts.forEach(x -> {
                    x.setPanelId(x.getId());
                    if (maps.containsKey(x.getId())) {
                        x.setId(maps.get(x.getId()));
                        baseMapper.updateById(x);
                    } else {
                        x.setId(IdWorker.get32UUID());
                        baseMapper.insert(x);
                    }
                });
            }
        }
    }

    @Override
    public List<InstrumentPanelDraft> getByIds(Set<String> panelIds) {
        LambdaQueryWrapper<InstrumentPanelDraft> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.in(InstrumentPanelDraft::getPanelId, panelIds);
        return baseMapper.selectList(where);
    }

    @Override
    public List<InstrumentPanelDraft> selectDraftList(CurrentUser currentUser,String appId, Set<String> panelIds, String officeIds, String userId, String roleId) {
        if (currentUser.getAdmin()){
            return baseMapper.selectDraftListByAdmin(appId,panelIds);
        }else {
            return baseMapper.selectDraftList(appId, panelIds, officeIds, userId, roleId);
        }
    }

    @Override
    public InstrumentPanelDraft getByPanelId(String panelId) {
        List<InstrumentPanelDraft> drafts = getByIds(Sets.newHashSet(panelId));
        return CollectionUtils.isNotEmpty(drafts)?drafts.get(0):null;
    }
}
