package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelGroupMapper;
import com.syc.slm.slmbi.dto.InstrumentPanelGroupDTO;
import com.syc.slm.slmbi.entity.BaseEntity;
import com.syc.slm.slmbi.entity.InstrumentPanelGroup;
import com.syc.slm.slmbi.entity.InstrumentPanelTree;
import com.syc.slm.slmbi.enums.NodeType;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.function.Action2;
import com.syc.slm.slmbi.service.InstrumentPanelGroupService;
import com.syc.slm.slmbi.service.InstrumentPanelService;
import com.syc.slm.slmbi.service.InstrumentPanelTreeService;
import com.syc.slm.slmbi.vo.InstrumentPanelGroupVo;
import com.syc.slm.slmbi.vo.PanelTreeNodeVo;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * slm-bi
 * 仪表板分组业务实现类
 *
 * @author : lin
 * @date : 2021-03-25 15:43
 **/
@Slf4j
@Service
public class InstrumentPanelGroupServiceImpl extends ServiceImpl<InstrumentPanelGroupMapper, InstrumentPanelGroup>
    implements InstrumentPanelGroupService {

    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private InstrumentPanelService panelService;

    @Override
    @Transactional(readOnly = true)
    public List<InstrumentPanelGroupVo> selectGroupList(String appId,String name) {
        LambdaQueryWrapper<InstrumentPanelGroup> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getAppId, appId);
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.orderByAsc(BaseEntity::getCreateTime);
        List<InstrumentPanelGroupVo> allGroups=baseMapper.selectGroupList(where);
        List<InstrumentPanelGroupVo> filteredGroups=allGroups;
        Map<String, InstrumentPanelGroupVo> allMap=allGroups.stream().collect(Collectors.toMap(InstrumentPanelGroupVo::getId, x->x));
        Map<String, InstrumentPanelGroupVo> filteredMap=allMap;
        where.like(InstrumentPanelGroup::getName, name);
        if(StringUtils.isNotEmpty(name)){
            filteredGroups=baseMapper.selectGroupList(where);
            filteredMap=filteredGroups.stream().collect(Collectors.toMap(InstrumentPanelGroupVo::getId, x->x));
        }
        Stack<InstrumentPanelGroupVo> stack=new Stack<>();
        stack.addAll(filteredGroups);
        while(!stack.isEmpty()){
            InstrumentPanelGroupVo group = stack.pop();
            String parentId=group.getParentId();
            if(StringUtils.isNotEmpty(parentId)) {
                InstrumentPanelGroupVo pGroup = Optional.ofNullable(filteredMap.get(parentId)).orElse(allMap.get(parentId));
                filteredMap.putIfAbsent(parentId, pGroup);
                stack.add(pGroup);
            }
        }
        return Lists.newArrayList(filteredMap.values()).stream().sorted(Comparator.comparing(InstrumentPanelGroupVo::getCreateTime)).collect(
            Collectors.toList());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String delGroup(@NonNull String groupId, boolean isDelTenant) {
        InstrumentPanelGroup group = baseMapper.selectById(groupId);
        if (group.getRecordStatus().equals(SlmConstants.DATA_DELETED)) {
            throw new SysException("该分组已经被删除，请重新刷新页面进行操作!");
        }
        if(!isDelTenant) {
            if (StringUtils.isEmpty(group.getParentId())) {
                throw new SysException("顶级分组不可删除");
            }
        }
        group.setRecordStatus(SlmConstants.DATA_DELETED);
        baseMapper.updateById(group);

        log.info("删除仪表板树本身");
        InstrumentPanelTree tree = treeService.getById(groupId);
        tree.setRecordStatus(SlmConstants.DATA_DELETED);
        treeService.updateById(tree);
        log.info("查询该仪表板树下的子节点");
        List<InstrumentPanelTree> children = getInstrumentPanelTrees.apply(tree.getFullId());

        if (CollectionUtils.isNotEmpty(children)) {
            Set<String> childrenIds = children.stream().map(BaseEntity::getId).collect(Collectors.toSet());

            log.info("删除该仪表板树下的子节点");
            treeService.batchUpdateTreeByIds(childrenIds, SlmConstants.DATA_DELETED);

            log.info("删除子分组");
            delChildrenGroups.accept(children);

            log.info("删除各个分组下的仪表板");
            delPanelByIds.accept(children);
        }
        return groupId;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String updateGroup(@NonNull InstrumentPanelGroupDTO group,String appId) {
        if(StringUtils.isEmpty(group.getName())){
            throw new SysException("仪表板分组名称必填");
        }
        if(StringUtils.isEmpty(group.getParentId())){
            throw new SysException("父级分组必填");
        }
        log.info("检查分组名字是否重复");
        checkedRepeatName.call(group,appId);
        InstrumentPanelGroup entity = new InstrumentPanelGroup();
        BeanUtils.copyProperties(group, entity);
        baseMapper.updateById(entity);
        log.info("更新分组在仪表盘树中的全路径");
        InstrumentPanelTree tree = treeService.getById(entity.getId());
        String beforeFullName = tree.getFullName();
        String beforeFullId = tree.getFullId();
        InstrumentPanelTree panelTree = treeService.getById(group.getParentId());
        tree.setFullName(panelTree.getFullName() + "." + entity.getName());
        tree.setFullId(panelTree.getFullId() + "." + entity.getId());
        tree.setName(entity.getName());
        treeService.updateById(tree);

        log.info("重新构建该分组下的仪表板树全路径");
        List<InstrumentPanelTree> children = getInstrumentPanelTrees.apply(beforeFullId);
        children.forEach(x -> {
            x.setFullId(x.getFullId().replace(beforeFullId, tree.getFullId()));
            x.setFullName(tree.getFullName()+"." + x.getFullName()
                .substring(beforeFullName.length()+1));
            treeService.updateById(x);
        });

        return tree.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveGroup(@NonNull InstrumentPanelGroupDTO group,String appId) {
        if(StringUtils.isEmpty(group.getName())){
            throw new SysException("仪表板分组名称必填");
        }
        log.info("校验名字是否重复");
        checkedRepeatName.call(group,appId);
        log.info("新增仪表板分组");
        InstrumentPanelGroup entity = new InstrumentPanelGroup();
        BeanUtils.copyProperties(group, entity);
        entity.setId(IdWorker.get32UUID());
        baseMapper.insert(entity);
        log.info("构建仪表板树 聚合表");
        InstrumentPanelTree tree = new InstrumentPanelTree();
        tree.setId(entity.getId());
        if (StringUtils.isNotEmpty(group.getParentId())) {
            InstrumentPanelTree panelTree = treeService.getById(group.getParentId());
            if (ObjectUtils.isEmpty(panelTree)){
                throw new SysException(group.getParentId() + ",上级不存在");
            }
            if (SlmConstants.DATA_DELETED.equals(panelTree.getRecordStatus())) {
                throw new SysException(panelTree.getName() + ",已不存在，请刷新页面");
            }
            tree.setFullId(panelTree.getFullId() + "." + entity.getId());
            tree.setFullName(panelTree.getFullName() + "." + entity.getName());
        } else {
            //顶级分组
            log.info("重新构建该分组下的仪表板树全路径");
            tree.setFullId(entity.getId());
            tree.setFullName(entity.getName());
        }
        tree.setNodeType(NodeType.GROUP.value);
        tree.setName(entity.getName());
        treeService.save(tree);
        return entity.getId();
    }

    @Override
    public List<PanelTreeNodeVo> selectGroupByIds(Set<String> finalPanelIds,String name) {
        return baseMapper.selectGroupByIds(finalPanelIds,name);
    }

    private Action2<InstrumentPanelGroupDTO,String> checkedRepeatName=(group,appId)-> {
        LambdaQueryWrapper<InstrumentPanelGroup> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(InstrumentPanelGroup::getName, group.getName());
        wrapper.eq(BaseEntity::getAppId, appId);
        wrapper.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        if (StringUtils.isNotEmpty(group.getId())) {
            wrapper.ne(InstrumentPanelGroup::getId, group.getId());
        }
        if (baseMapper.selectCount(wrapper) > 0) {
            throw new SysException("仪表板分组名字:" + group.getName() + "已存在!");
        }
    };


    private Consumer<List<InstrumentPanelTree>> delChildrenGroups = (trees) -> {
        Set<String> groupIds = trees.stream()
            .filter(x -> NodeType.GROUP.value.equals(x.getNodeType()))
            .map(BaseEntity::getId)
            .collect(Collectors.toSet());
        if (CollectionUtils.isNotEmpty(groupIds)) {
            if (CollectionUtils.isNotEmpty(groupIds)) {
                baseMapper.batchUpdateGroupByIds(groupIds, SlmConstants.DATA_DELETED);
            }
        }
    };

    private Consumer<List<InstrumentPanelTree>> delPanelByIds = (trees) -> {
        Set<String> panelIds = trees.stream()
            .filter(x -> NodeType.PANEL.value.equals(x.getNodeType()))
            .map(BaseEntity::getId)
            .collect(Collectors.toSet());
        if(CollectionUtils.isNotEmpty(panelIds)) {
            panelService.batchUpdatePanelByIds(panelIds, SlmConstants.DATA_DELETED);
        }
    };

    private Function<String, List<InstrumentPanelTree>> getInstrumentPanelTrees = (fullId) -> {
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.likeRight(InstrumentPanelTree::getFullId, fullId + ".");
        return treeService.list(where);
    };

}
